#!/usr/bin/python
# -*- coding:utf-8 -*-
"""
Download TV program table from tv.goo.ne.jp and convert it to csv file.

Usage : python loadtv2csv.py [ -area (prefecture) ]
                             [ -wave (am/fm/vhf/uhf/bs_digital/4thMEDIA_1) ]
                             [ -offset <number> ]
                             [ -o <output filename> ]
080412062007 - revised by 367
"""

import os, string, re, sys

class Program:
    """Television program containing time, title, description, genre and link.
    """
    # static regular expressions
    regex_time = re.compile( 'valign="top"><strong>(\\d\\d:\\d\\d)</strong>' )
    regex_title = re.compile( 'index.html"><strong>(.*?)</strong></a>' )
    regex_rmtag = re.compile( "<.*?>" )
    regex_rdbr  = re.compile( "[<br>]+" )
    regex_addbr = re.compile( "▽" )
    regex_img   = re.compile( "<img\\s.*?alt=['\"]?(.*?)['\"]?[\\s>][^>]*>" )

    def __init__( self, area, data ):
        data = data.strip()
        self.area = area
        self.comment = ""
        self.iepg = ""
        self.genre = 0

        mt = self.regex_time.search( data )
        self.time = mt.group( 1 )
        hr, mn = self.time.split( ':' )

        self.startTime = string.atoi( hr ) * 60 + string.atoi( mn )

        ml = self.regex_title.search( data, mt.end() )
        self.title = ml.group( 1 )

        self.gcode = ''
        # convert icons to abbreviations
        mi = self.regex_img.search( self.title )
        if mi: 
            while mi:
                alt = mi.group( 1 )
                if alt == "新番組" : alt = "[新]"
                elif alt == "最終回" : alt = "[最]"
                elif alt == "再放送" : alt = "[再]"
                elif alt == "初回放送" : alt = "[初]"
                elif alt == "前編" : alt = "[前]"
                elif alt == "後編" : alt = "[後]"
                elif alt == "映画" : alt = "[映]"
                elif alt == "ニュース" : alt = "[N]"
                elif alt == "天気" : alt = "[天]"
                elif alt == "交通情報" : alt = "[交]"
                elif alt == "出演" : alt = "[出]"
                elif alt == "ゲスト" : alt = "[ゲ]"
                elif alt == "司会" : alt = "[司]"
                elif alt == "声の出演" : alt = "[声]"
                elif alt == "解説" : alt = "[解]"
                elif alt == "監督" : alt = "[監]"
                elif alt == "脚本家" : alt = "[脚]"
                elif alt == "原作" : alt = "[原]"
                elif alt == "語り" : alt = "[語]"
                elif alt == "実況" : alt = "[実]"
                elif alt == "演出者" : alt = "[演]"
                elif alt == "ステレオ" : alt = "[ス]"
                elif alt == "サラウンドステレオ" : alt = "[サ]"
                elif alt == "Bモードステレオ" : alt = "[B]"
                elif alt == "音声多重放送" : alt = "[音]"
                elif alt == "文字放送" : alt = "[文]"
                elif alt == "データ放送" : alt = "[デ]"
                elif alt == "双方向放送" : alt = "[双]"
                elif alt == "手話" : alt = "[手]"
                elif alt == "二ヶ国語放送" : alt = "[二]"
                elif alt == "吹き替え" : alt = "[吹]"
                elif alt == "クローズドキャプション" : alt = "[ク]"
                elif alt == "ワイドビジョン" : alt = "[ワ]"
                elif alt == "ハイビジョン" : alt = "[ハ]"
                elif alt == "SDTV1" : alt = "[S]"
                elif alt == "ペイパービュー" : alt = "[ペ]"
                elif alt == "マルチビュー" : alt = "[マ]"
                self.title = self.title[ 0: mi.start() ] + alt + self.title[ mi.end(): ]
                mi = self.regex_img.search( self.title )
                pass
            pass

        temp_description = self.regex_rmtag.sub( "<br>", data[ ml.end(): ] ).strip()
        temp_description = self.regex_addbr.sub("<br>▽",temp_description)
        self.description = self.regex_rdbr.sub("<br>",temp_description)
        
        self.span = 10# dummy
        pass

    def setID( self, idnum ):
        self.idvalue = idnum
        pass

    def getID( self ):
        return self.idvalue

    def setGenre( self, genre ):
        self.genre = genre
        pass

    def setSpan( self, span ):
        if span < 0: span += 1440
        self.span = span
        pass

    def setIepg( self, iepg ):
        self.iepg = "http://tv.goo.ne.jp" + iepg
        pass

    def setGCode( self, code ):
        self.gcode = code
        pass

    def __repr__( self ):
        output = "%d,%d,%s,%s,%s,%d,%s,%s" % ( self.startTime, self.span, self.title, self.description, self.comment, Goo.goo_genre[self.genre], self.gcode, self.iepg )
        return output
    pass

class ProgramTable:
    """Programs of the day. The objects contains a sorted array of Porgram objects and
    name of the channel."""
    def __init__( self, progs = None ):
        self.name = ""
        if progs == None:
            self.programs = []
        else:
            self.programs = progs
        pass

    def set_name( self, name ):
        self.name = name
        pass

    def get_name( self ):
        return self.name

    def append( self, prog ):
        self.programs.append( prog )
        pass


class Goo:
# just to keep the script somewhat convertible to previous one.
    goo_area = {
        "hokk"      :   "001",
        "aomon"     :   "002",
        "akita"     :   "003",
        "iwate"     :   "004",
        "yamagata"  :   "005",
        "miya"      :   "006",
        "fuku"      :   "007",
        "tokyo"     :   "008",
        "kanagawa"  :   "009",
        "saitama"   :   "010",
        "chiba"     :   "011",
        "gunma"     :   "012",
        "tochigi"   :   "013",
        "ibaraki"   :   "014",
        "niigata"   :   "015",
        "nagano"    :   "016",
        "yamanasi"  :   "017",
        "sizu"      :   "018",
        "aichi"     :   "019",
        "gifu"      :   "020",
        "mie"       :   "021",
        "toyama"    :   "022",
        "ishikawa"  :   "023",
        "fukui"     :   "024",
        "osaka"     :   "025",
        "kyoto"     :   "026",
        "hyougo"    :   "027",
        "nara"      :   "028",
        "wakayama"  :   "029",
        "siga"      :   "030",
        "okayama"   :   "031",
        "hirosima"  :   "032",
        "tottori"   :   "033",
        "shimane"   :   "034",
        "yamaguchi" :   "035",
        "kagawa"    :   "036",
        "tokusima"  :   "037",
        "ehime"     :   "038",
        "kouchi"    :   "039",
        "fukuoka"   :   "040",
        "saga"      :   "041",
        "nagasaki"  :   "042",
        "kumamoto"  :   "043",
        "oita"      :   "044",
        "miyazaki"  :   "045",
        "kagosima"  :   "046",
        "nawa"      :   "047",
    }
    goo_wave = {
        'am'    :   'am',
        'fm'    :   'fm',
        'vhf'   :   "VHF_1",
        'uhf'   :   "UHF-BSanalog_1",
        'bs_digital':   "BSdigital_1",
        '4thMEDIA_1':   "4thMEDIA_1",
    }
    goo_genre = {
        "genre_house"   :   0,
        "genre_news"    :   1,
        "genre_drama"   :   2,
        "genre_variety" :   7,
        "genre_movie"   :   6,
        "genre_music"   :   5,
        "genre_anime"   :   4,
        "genre_sport"   :   3,
        "genre_XX"      :   8,
    }

if __name__ == "__main__":
    import urllib, tempfile, time

    # read arguments
    use_urllib = False
    file_out = sys.stdout
    filename_output = None
    flag_radio = False
    area = 'tokyo'
    wave = 'vhf'
    offset = 0

    i = 1
    while i < len( sys.argv ) - 1:
        ar = sys.argv[ i ]
        if ar == '-o':
            filename_output = sys.argv[ i + 1 ]
            pass
        elif ar == '-area':
            area = sys.argv[ i + 1 ]
        elif ar == '-wave':
            wave = Goo.goo_wave[ sys.argv[ i + 1 ] ]
            pass
        elif ar == '-offset':
            try:
                offset = string.atoi( sys.argv[ i + 1 ] )
            except:
                pass
            pass
        i += 1
        pass

    if filename_output:
        file_out = open( filename_output, "w" )
        pass

    if wave == 'UHF-BSanalog_1':
        baseurl = "http://tv.goo.ne.jp/contents/epg/%s/UHF-BSanalog_1/today_0024/index.html"
    elif wave == 'BSdigital_1':
        baseurl = "http://tv.goo.ne.jp/contents/epg/%s/BSdigital_1/today_0024/index.html"
    elif wave == '4thMEDIA_1':
        baseurl = "http://tv.goo.ne.jp/contents/epg/%s/4thMEDIA_1/today_0024/index.html"
    elif wave == 'am':
        flag_radio = True
        baseurl = "http://tv.yahoo.co.jp/radio/%s/%s24.html"
    elif wave == 'fm':
        flag_radio = True
        baseurl = "http://tv.yahoo.co.jp/radio/%s/%s24fm.html"
    else:
        baseurl = url = "http://tv.goo.ne.jp/contents/epg/%s/VHF_1/today_0024/index.html"
        pass
        
    # laod HTML
    lt = time.localtime( time.time() + offset * 3600 * 24 )
    today = "%d%s%s" % ( lt[ 0 ], ( "0%d" % lt[ 1 ] )[ -2: ], ( "0%d" % lt[ 2 ] )[ -2: ] )
    if wave == 'am' or wave == 'fm':
        url = baseurl % ( area, today )
    else:
        url = baseurl % ( Goo.goo_area[ area ] )

    print url
    if use_urllib:
        stream = urllib.urlopen( url )
        fn_tmp = tempfile.mktemp()
        open( fn_tmp, "w" ).write( stream.read() )
        stream.close()
    else:
        fn_tmp = tempfile.mktemp()
        command = "/usr/bin/curl \"%s\" --silent -o %s" % ( url, fn_tmp )
        #print command
        os.system( command )
        pass
    
    fn_utf = tempfile.mktemp()
#    my_dir = os.path.dirname(os.path.abspath(__file__))
    os.system( "/usr/bin/iconv -f euc-jp -t utf-8 %s > %s" % ( fn_tmp, fn_utf ) )
#    os.system( "%s/nkf --ic=eucJP-ms --oc=utf-8 %s > %s" % ( my_dir, fn_tmp, fn_utf ) )
    os.unlink( fn_tmp )
    
    # execute external script for radio programs
    if flag_radio:
        import r2c
        r2c.convert_radio_html_to_csv( fn_utf, file_out )
        file_out.close()
        os.unlink( fn_utf )
        sys.exit()
        pass

    # parse HTML
    fn = open( fn_utf )
    line = fn.readline()
    area_html = ""
    wave_html = ""
    date_html = ""
    # read title
    re_title = re.compile( '(^\\d.*)<a href="/rss' ) #"
    while line != "":
        m = re_title.search( line )
        if m:
            title = m.group( 1 )
            area_html = title
            m = re.search( "\[(.*?) - (.*?)\]", title )
            if m:
                area_html = m.group( 2 )
                wave_html = m.group( 1 )
                pass
            m = re.search( "(\\d+)月(\\d+)日", title )
            if m:
                date_html = m.group( 1 ) + "/" + m.group( 2 )
                pass
            break
        line = fn.readline()
        pass

    # read channel names
    channels = {}
    channel_order = []
    flag_start = False
    r_channel = re.compile( '<a href="/contents/station/\\d+/.*?/(.*)/0024/index.html">(.*)</a>' )
    while line != "":
        line = line.strip()
        if line == "<!--ch_top-->" : flag_start = True
        if line == "<!--/ch_top-->": break
        if flag_start:
            #print line
            m_channel = r_channel.search( line )
            if m_channel:
                channels[ m_channel.group( 1 ) ] =  m_channel.group( 2 )
                channel_order.append( m_channel.group( 1 ) )
                pass
            pass
        line = fn.readline()
        pass

    # read programs
    programs = {}
    storedorder = []
    r_tdclass = re.compile( 'td class="(.*?)"' )
    r_prog = re.compile( 'href="/contents/program/\\d+/(.*?)/index.html"><strong>' )
    r_gcode = re.compile( "<br>(\\d+)</td></tr?" )
    r_iepg = re.compile( "href=\"(/contents/program/\\d+/\\d+/\\d+_\\d+/iepg.tvpi)\"" )
    genre = Goo.goo_genre['genre_house']
    p = None

    while line != "":
        inlines = line.split('</table></td>')
        for inline in inlines:
            m_tdclass = r_tdclass.search( inline )
            if m_tdclass:
                # genre
                tdclass = m_tdclass.group( 1 )
                genre = tdclass
                pass
            # program
            m_prog = r_prog.search( inline )
            if m_prog:
                p = Program( Goo.goo_area[ area ], inline )
                p.setGenre( genre )
                idval = m_prog.group( 1 )
                p.setID( idval )
                programs[ idval ] = p
                pass
            if r_gcode and p:
                # gcode
                m_gcode = r_gcode.search( line )
                if m_gcode:
                    p.setGCode( m_gcode.group( 1 ) )
                    pass
                # iepg
                m_iepg = r_iepg.search( line )
                if m_iepg:
                    p.setIepg( m_iepg.group( 1 ) )
                    pass
                pass
            pass
        line = fn.readline()
        pass
    fn.close()

    # output CSV file
    keys = programs.keys()
    keys.sort()
    r_theday = re.compile( "/(\\d+)_" )

    for ch in range( 0, len(channel_order) ):
        for pg in range( 0, len( keys ) ):
            if ( pg + 1 )  < len( keys ):
                if (keys[ pg ].find( channel_order[ ch ] ) == 0) and (keys[ pg + 1 ].find( channel_order[ ch ] ) == 0): 
                    span = programs[ keys[ pg + 1 ] ].startTime - programs[ keys[ pg ] ].startTime
                    programs[ keys[ pg ] ].setSpan( span )

    file_out.write( "#%s,%s,%s\n" % ( area_html, date_html, wave_html ) )
    for ch in channel_order:
        file_out.write( "*%s\n" % ( channels[ ch ] ) )
        for pg in keys:
            if pg.find( ch ) == 0:
                m_theday = r_theday.search( pg )
                theday = m_theday.group( 1 )
                if today < theday: programs[ pg ].startTime += 1440
                file_out.write( "%s\n" % repr( programs[ pg ] ) )
    file_out.close()
    os.unlink( fn_utf )
    pass
